/*
 * Project name:
     Rs485_Master_Example (RS485 Library demo - Master side)
 * Copyright:
     (c) MikroElektronika, 2005.
 * Description:
     This is a simple demonstration on how to use the mikroC's RS485 library.
     It is being used in pair with the Rs485_Slave_Example project. Master (this
     machine) initiates communication with slave by sending 1 byte of data to
     the slave with designated slave address (160). The slave accepts data,
     increments it and sends it back to the master.
     The data received is shown on PORTB, Error on receive (0xAA) and number of
     consecutive unsuccessful retries on PORTD.
     Several situations are shown here:
     - RS485 Master Init sequence;
     - Data sending master-to-slave with designated slave address;
     - Data sending master-to-slave with broadcast slave address (50);
     - Handling of unsuccessful master-slave communication (connection reset);
     
     Also shown here, but not strictly related to RS485 library, is:
     - Function calling from the interrupt routine - which data is to be saved,
       and how.
     For further explanations on RS485 library, please consult the mikroC Help.
     
 * Test configuration:
     MCU:             PIC16F877A
     Dev.Board:       EasyPIC2
     Oscillator:      External, 8.000 MHz
     Ext. Modules:    RS485 on PORTC
     SW:              mikroC v2.0.0.2
 * NOTES:
     - Be careful to initialize the USART module before performing RS485 init!
     - RS485 module is connected to PORTC, i.e. where the USART module is.
*/

char dat[10];             // buffer for receving/sending messages
char i,j, Iflag;

char bs[10];              // buffer for backup of variables during interrupt call

//-------------- Interrupt routine
void interrupt(){
 bs[0] = 0;
 asm{                                // Save all the necessary variables
   MOVF      STACK_0,W
   MOVWF     INDF
   INCF      FSR,F
   MOVF      STACK_1,W
   MOVWF     INDF
   INCF      FSR,F
   MOVF      STACK_2,W
   MOVWF     INDF
   INCF      FSR,F
   MOVF      STACK_3,W
   MOVWF     INDF
   INCF      FSR,F
   MOVF      STACK_4,W
   MOVWF     INDF
   INCF      FSR,F
   MOVF      STACK_5,W
   MOVWF     INDF
   INCF      FSR,F
   MOVF      STACK_6,W
   MOVWF     INDF
   INCF      FSR,F
   MOVF      STACK_7,W
   MOVWF     INDF
   INCF      FSR,F
   BCF	     STATUS,RP1
   BCF	     STATUS,RP0
   MOVF      RS485master_receive_argh0,W
   MOVWF     INDF
   INCF      FSR,F
   BCF	     STATUS,RP1
   BCF	     STATUS,RP0
   MOVF      Usart_Read_tmp_L0,W
   MOVWF     INDF
 }
 asm {
   MOVLW	 _dat_1
   BCF     STATUS,RP1                // Set the bank for function argument
   BCF	   STATUS,RP0
   MOVWF   RS485master_receive_argh0
   CALL 	 _RS485master_receive      //    and call the function from assembler
 }
 (char)bs[0];
 asm{                                // Restore variables
   MOVF    INDF,W
   MOVWF   STACK_0
   INCF    FSR,F
   MOVF    INDF,W
   MOVWF   STACK_1
   INCF    FSR,F
   MOVF    INDF,W
   MOVWF   STACK_2
   INCF    FSR,F
   MOVF    INDF,W
   MOVWF   STACK_3
   INCF    FSR,F
   MOVF    INDF,W
   MOVWF   STACK_4
   INCF    FSR,F
   MOVF    INDF,W
   MOVWF   STACK_5
   INCF    FSR,F
   MOVF    INDF,W
   MOVWF   STACK_6
   INCF    FSR,F
   MOVF    INDF,W
   MOVWF   STACK_7
   INCF    FSR,F
   MOVF    INDF,W
   BCF	   STATUS,RP1
   BCF	   STATUS,RP0
   MOVWF   RS485master_receive_argh0
   INCF    FSR,F
   MOVF    INDF,W
   BCF	   STATUS,RP1
   BCF	   STATUS,RP0
   MOVWF   Usart_Read_tmp_L0

 }
}//~


void main(){
  long you = 0;
  PORTA = 0x00;
  TRISA = 0;
  ADCON1 = 6;
  portd = 0;
  trisd = 0;
  trisb = 0;
  USART_init(19200);                   // initialize usart module
  Delay_us(500);

  RS485master_init();                  // intialize mcu as master
  dat[0] = 0xAA;
  dat[1] = 0xF0;
  dat[2] = 0x0F;
  dat[5] = 0;
  dat[6] = 0;
  Iflag = 0;
  RS485master_send(dat,1,160);
  PORTB = 0;
  PORTD = 0;
  dat[4] = 0;                          // ensure that message received flag is 0
  dat[5] = 0;                          // ensure that error flag is 0

  PIE1.RCIE = 1;                       // enable interrupts
  INTCON.PEIE = 1;                     //   on byte received
  PIE2.TXIE = 0;                       //   via USART (RS485)
  INTCON.GIE = 1;

  while (1){
                                       // upon completed valid message receiving
                                       //   data[4] is set to 255
    you++;
    if (dat[5])  {                     // if an error detected, signal it
      PORTD = 0xAA;                    //   by setting portd to 0xAA
    }
    if (dat[4]) {                      // if message received successfully
      you = 0;
      dat[4] = 0;                      // clear message received flag
      j = dat[3];
      for (i = 1; i <= dat[3]; i++) {  // show data on PORTB
        PORTB = dat[i-1];
      }                                // increment received dat[0]
      dat[0] = dat[0]+1;               // send back to master
      Delay_ms(500);
      RS485master_send(dat,1,160);
    }
   if (you > 100000) {
      PORTD ++;
      you = 0;
      RS485master_send(dat,1,160);
      if (PORTD > 10)                  // if sending failed 10 times
        RS485master_send(dat,1,50);    //   send message on broadcast address

     }
  }
  
   if (0) {                            // This part of code will never be
                                       //   executed, but call to
     RS485master_receive(dat);         //   RS485master_receive() must be
                                       //   present in (non-asm) code for the
   }                                   //   function to be properly linked.

}//~!
