/*
 * Project name:
     Rs485_Slave_Example (RS485 Library demo - Slave side)
 * Copyright:
     (c) MikroElektronika, 2005.
 * Description:
     This is a simple demonstration on how to use the mikroC's RS485 library.
     It is being used in pair with the Rs485_Master_Example project. Slave (this
     machine) initializes itself (on address 160) and waits to receive data from
     the master. Then it increments the first byte of received data and sends it
     back to the master. The data received is shown on PORTB, Error on receive
     (0xAA) and number of consecutive unsuccessful retries on PORTD.
     Several situations are shown here:
     - RS485 Slave Init sequence;
     - Data sending slave-to-master;

     Also shown here, but not strictly related to RS485 library, is:
     - Function calling from the interrupt routine - which data is to be saved,
       and how.
     For further explanations on RS485 library, please consult the mikroC Help.

 * Test configuration:
     MCU:             PIC16F877A
     Dev.Board:       EasyPIC2
     Oscillator:      External, 8.000 MHz
     Ext. Modules:    RS485 on PORTC
     SW:              mikroC v2.0.0.2
 * NOTES:
     - Be careful to initialize the USART module before performing RS485 init!
     - RS485 module is connected to PORTC, i.e. where the USART module is.
*/

char dat[9];             // buffer for receving/sending messages
char i,j, Iflag;

char  bs[11];            // buffer for backup of variables during interrupt call

//-------------- Interrupt routine
void interrupt(){
 bs[0] = 0;
 asm{
   MOVF STACK_0,W             // Save all the necessary variables
   MOVWF INDF
   INCF FSR,F
   MOVF STACK_1,W
   MOVWF INDF
   INCF FSR,F
   MOVF STACK_2,W
   MOVWF INDF
   INCF FSR,F
   MOVF STACK_3,W
   MOVWF INDF
   INCF FSR,F
   MOVF STACK_4,W
   MOVWF INDF
   INCF FSR,F
   MOVF STACK_5,W
   MOVWF INDF
   INCF FSR,F
   MOVF STACK_6,W
   MOVWF INDF
   INCF FSR,F
   MOVF STACK_7,W
   MOVWF INDF
   INCF FSR,F
   MOVF STACK_8,W
   MOVWF INDF
   INCF FSR,F
   MOVF STACK_9,W
   MOVWF INDF
   INCF FSR,F
   BCF	STATUS,RP1
   BCF	STATUS,RP0
   MOVF RS485slave_receive_argh0,W
   MOVWF INDF
 }
 asm {

   MOVLW	  _dat_1
   BCF     STATUS,RP1                // Set the bank for function argument
   BCF	   STATUS,RP0
   MOVWF    RS485slave_receive_argh0
   CALL 	  _RS485slave_receive      //    and call the function from assembler
 }
 (char)bs[0];
 asm{                                // Restore variables
   MOVF INDF,W
   MOVWF STACK_0
   INCF FSR,F
   MOVF INDF,W
   MOVWF STACK_1
   INCF FSR,F
   MOVF INDF,W
   MOVWF STACK_2
   INCF FSR,F
   MOVF INDF,W
   MOVWF STACK_3
   INCF FSR,F
   MOVF INDF,W
   MOVWF STACK_4
   INCF FSR,F
   MOVF INDF,W
   MOVWF STACK_5
   INCF FSR,F
   MOVF INDF,W
   MOVWF STACK_6
   INCF FSR,F
   MOVF INDF,W
   MOVWF STACK_7
   INCF FSR,F
   MOVF INDF,W
   MOVWF STACK_8
   INCF FSR,F
   MOVF INDF,W
   MOVWF STACK_9
   INCF FSR,F
   MOVF INDF,W
   BCF	STATUS,RP1
   BCF	STATUS,RP0
   MOVWF RS485slave_receive_argh0
 }
}//~

void main() {
  TRISD = 0;
  PORTD = 0;
  TRISB = 0;
  USART_init(19200);                 // initialize usart module
  Rs485slave_Init(160);              // intialize mcu as slave, address 160
  dat[5] = 0;                        // ensure that message received flag is 0
  dat[6] = 0;                        // ensure that error flag is 0
  Iflag = 0;
  PIE1.RCIE = 1;                     // enable interrupts
  INTCON.PEIE = 1;                   //   on byte received
  PIE2.TXIE = 0;                     //   via USART (RS485)
  INTCON.GIE = 1;
  PORTB = 0;
  PORTD = 0;
  dat[4] = 0;                        // ensure that message received flag is 0
  dat[5] = 0;                        // ensure that error flag is 0
  
  while (1) {
    if (dat[5])  {                   // if an error detected, signal it by
      PORTD = 0xAA;                  //   setting portd to 0xAA
    }
    if (dat[4]) {                    // upon completed valid message receive
      dat[4] = 0;                    //   data[4] is set to 0xFF
      j = dat[3];
      for (i = 1; i <= dat[3];i++){
        PORTB = dat[i-1];
        Delay_ms(500);
      }
      dat[0] = dat[0]+1;             // increment received dat[0]
      Delay_ms(50);
      Rs485slave_Send(dat,1);        //   and send it back to master
    }
  }
  if (0) {                           // This part of code will never be
    Rs485slave_Receive(dat);         //   executed, but call to
                                     //   Rs485slave_Receive() must be
                                     //   present in (non-asm) code for the
   }                                 //   function to be properly linked.
}//~!
